from __future__ import unicode_literals
from __future__ import print_function

import re
import unittest

from mock import patch

from tcell_agent.appsensor.meta import AppSensorMeta
from tcell_agent.appsensor.params import GET_PARAM
from tcell_agent.config import CONFIGURATION
from tcell_agent.policies.appsensor.payloads_policy import PayloadsPolicy


class PayloadsPolicyTest(unittest.TestCase):

    def false_send_payloads_apply_test(self):
        policy_json = { "payloads": {"send_payloads": False} }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = False

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertIsNone(payload)
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_false_allow_unencrypted_appfirewall_payloads_apply_test(self):
        policy_json = { "payloads": {"send_payloads": True} }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = False

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertIsNone(payload)
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_true_allow_unencrypted_appfirewall_payloads_no_blacklist_no_whitelist_apply_test(self):
        policy_json = { "payloads": {"send_payloads": True, "send_blacklist": {}} }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = True

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertEquals(payload, '123password123')
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_true_allow_unencrypted_appfirewall_payloads_no_blacklist_with_whitelist_missing_param_apply_test(self):
        policy_json = { "payloads": {"send_payloads": True, "send_blacklist": {}, "send_whitelist": {"username": ["*"]}} }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = True

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertEquals(payload, 'NOT_WHITELISTED')
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_true_allow_unencrypted_appfirewall_payloads_no_blacklist_param_in_whitelist_but_incorrect_location_apply_test(self):
        policy_json = {
            "payloads": {
                "send_payloads": True,
                "send_blacklist": {},
                "send_whitelist": {
                    "username": ["*"],
                    "password": ["cookie"]
                }
            }
        }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = True

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertEquals(payload, 'NOT_WHITELISTED')
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_true_allow_unencrypted_appfirewall_payloads_no_blacklist_param_in_whitelist_apply_test(self):
        policy_json = {
            "payloads": {
                "send_payloads": True,
                "send_blacklist": {},
                "send_whitelist": {
                    "username": ["*"],
                    "password": ["form"]
                }
            }
        }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = True

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertEquals(payload, '123password123')
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_true_allow_unencrypted_appfirewall_payloads_no_whitelist_with_blacklist_missing_param_apply_test(self):
        policy_json = { "payloads": {"send_payloads": True, "send_blacklist": {"username": ["*"]}} }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = True

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertEquals(payload, '123password123')
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_true_allow_unencrypted_appfirewall_payloads_no_whitelist_param_in_blacklist_but_incorrect_location_apply_test(self):
        policy_json = {
            "payloads": {
                "send_payloads": True,
                "send_blacklist": {
                    "username": ["*"],
                    "password": ["cookie"]
                }
            }
        }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = True

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertEquals(payload, '123password123')
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')

    def true_send_payloads_true_allow_unencrypted_appfirewall_payloads_no_whitelist_param_in_blacklist_apply_test(self):
        policy_json = {
            "payloads": {
                "send_payloads": True,
                "send_blacklist": {
                    "username": ["*"],
                    "password": ["form"]
                }
            }
        }
        policy = PayloadsPolicy()
        policy.from_json(policy_json)
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )

        old_allow = CONFIGURATION.allow_unencrypted_appsensor_payloads
        CONFIGURATION.allow_unencrypted_appsensor_payloads = True

        with patch.object(PayloadsPolicy, 'log', return_value=None) as patched_log:
            payload = policy.apply("xss", meta, GET_PARAM, "password", "123password123", {"l": "query"}, "pattern")

            CONFIGURATION.allow_unencrypted_appsensor_payloads = old_allow

            self.assertEquals(payload, 'BLACKLISTED')
            patched_log.assert_called_once_with(
                'xss',
                meta,
                GET_PARAM,
                'password',
                '123password123',
                {'l': 'query'},
                'pattern')



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function

from tcell_agent.agent import TCellAgent, PolicyTypes

import logging
logger = logging.getLogger('tcell_agent').getChild(__name__)
def _instrument():
    old_func = Flask.__init__
    def init(*args):
        TCellAgent.get_agent().ensure_polling_thread_running()
        return old_func(*args)
    Flask.__init__ = init

    old_process_response = Flask.process_response
    def process_response(self, response):
        result = old_process_response(self, response)
        csp_headers = TCellAgent.get_policy(PolicyTypes.CSP).headers()
        if csp_headers:
            for csp_header in csp_headers:
                response.headers[csp_header[0]] = csp_header[1]
        return result
    Flask.process_response = process_response

try:
    from flask.app import Flask
    if TCellAgent.tCell_agent:
        _instrument()
except Exception as e:
    logger.debug("Could not instrument flask")



from __future__ import unicode_literals
from __future__ import print_function

import json
import unittest

from mock import call, patch
from django.utils.datastructures import MultiValueDict
from tcell_agent.appsensor.meta import AppSensorMeta
from tcell_agent.appsensor.params import POST_PARAM
from tcell_agent.policies.appsensor_policy import AppSensorPolicy

class FakeFile(object):
    def __init__(self, filename):
        self.name = filename

class AppSensorPolicyCheckParamsTest(unittest.TestCase):


    def uploading_zero_file_test(self):
        policy = AppSensorPolicy()
        files_dict = MultiValueDict()
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )
        meta.request_data(1024, {}, {}, {}, '', 'user_agent', files_dict)

        with patch('tcell_agent.policies.appsensor_policy.safe_wrap_function') as patched_safe_wrap_function:
            policy.check_params_for_injections(meta, {})
            self.assertFalse(patched_safe_wrap_function.called)

    def uploading_one_file_test(self):
        policy = AppSensorPolicy()
        files_dict = MultiValueDict({'avatar': [FakeFile('potention_injection')]})
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )
        meta.request_data(1024, {}, {}, {}, '', 'user_agent', files_dict)

        with patch('tcell_agent.policies.appsensor_policy.safe_wrap_function') as patched_safe_wrap_function:
            policy.check_params_for_injections(meta, {})
            patched_safe_wrap_function.assert_called_once_with(
                'Check Filename injections',
                policy.check_param_for_injections,
                POST_PARAM,
                meta,
                'avatar',
                'potention_injection'
            )

    def uploading_two_files_for_same_param_test(self):
        policy = AppSensorPolicy()
        files_dict = MultiValueDict({
            'avatar': [FakeFile('potention_injection'), FakeFile('second_potention_injection')]
        })
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )
        meta.request_data(1024, {}, {}, {}, '', 'user_agent', files_dict)

        with patch('tcell_agent.policies.appsensor_policy.safe_wrap_function') as patched_safe_wrap_function:
            policy.check_params_for_injections(meta, {})
            patched_safe_wrap_function.assert_has_calls([
                call(
                    'Check Filename injections',
                    policy.check_param_for_injections,
                    POST_PARAM,
                    meta,
                    'avatar',
                    'potention_injection'),
                call(
                    'Check Filename injections',
                    policy.check_param_for_injections,
                    POST_PARAM,
                    meta,
                    'avatar',
                    'second_potention_injection')
            ], True)


    def uploading_two_files_for_different_params_test(self):
        policy = AppSensorPolicy()
        files_dict = MultiValueDict({
            'avatar': [FakeFile('potention_injection')],
            'picture': [FakeFile('second_potention_injection')]
            })
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )
        meta.request_data(1024, {}, {}, {}, '', 'user_agent', files_dict)

        with patch('tcell_agent.policies.appsensor_policy.safe_wrap_function') as patched_safe_wrap_function:
            policy.check_params_for_injections(meta, {})
            patched_safe_wrap_function.assert_has_calls(
                [
                    call(
                        'Check Filename injections',
                        policy.check_param_for_injections,
                        POST_PARAM,
                        meta,
                        'avatar',
                        'potention_injection'),
                    call(
                        'Check Filename injections',
                        policy.check_param_for_injections,
                        POST_PARAM,
                        meta,
                        'picture',
                        'second_potention_injection')
                ],
                True
            )



# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function
from . import SensorEvent

"""
Use this to cause the event queue to flush if you don't need to send an actual event (ie, metrics get too big)
"""
class FlushDummyEvent(SensorEvent):
    def __init__(self):
        super(FlushDummyEvent, self).__init__("dummy", flush_right_away=True, send_event=False)



import json
import re

from future.utils import iteritems
from tcell_agent.appsensor.params import GET_PARAM, POST_PARAM, JSON_PARAM, COOKIE_PARAM, URI_PARAM
from tcell_agent.appsensor.rules import rule_manager
from tcell_agent.config import CONFIGURATION
from tcell_agent.policies.appsensor.sensor import sendEvent
from tcell_agent.sensor_events import AppSensorEvent


class InjectionSensor(object):

    PARAM_TYPE_TO_L = {
        GET_PARAM: 'query',
        POST_PARAM: 'body',
        JSON_PARAM: 'body',
        URI_PARAM: 'uri',
        COOKIE_PARAM: 'cookie'
    }

    def __init__(self, dp, policy_json=None):
        self.enabled = False
        self.dp = dp
        self.exclude_headers = False
        self.exclude_forms = False
        self.exclude_cookies = False
        self.exclusions = {}
        self.active_pattern_ids = {}
        self.v1_compatability_enabled = False
        self.excluded_route_ids = {}

        if policy_json is not None:
            self.enabled = policy_json.get("enabled", False)
            self.exclude_headers = policy_json.get("exclude_headers", False)
            self.exclude_forms = policy_json.get("exclude_forms", False)
            self.exclude_cookies = policy_json.get("exclude_cookies", False)
            self.v1_compatability_enabled = policy_json.get("v1_compatability_enabled", False)

            for route_id in policy_json.get("exclude_routes", []):
                self.excluded_route_ids[route_id] = True

            for pattern in policy_json.get("patterns", []):
                self.active_pattern_ids[pattern] = True

            for common_word, locations in iteritems(policy_json.get("exclusions", {})):
                self.exclusions[common_word] = locations

    def check(self, type_of_param, appsensor_meta, param_name, param_value, payloads_policy):
        if not self.enabled:
            return False

        if self.excluded_route_ids.get(appsensor_meta.route_id, False):
            return False

        if self.exclude_forms and (GET_PARAM == type_of_param or POST_PARAM == type_of_param or JSON_PARAM == type_of_param):
            return False

        if self.exclude_cookies and COOKIE_PARAM == type_of_param:
            return False

        vuln_results = self.check_dp_violation(param_name, param_value)

        if vuln_results:
            vuln_param = vuln_results.get("param")

            if vuln_param:
                meta = {"l": self.PARAM_TYPE_TO_L[type_of_param]}
                pattern = vuln_results.get("pattern")

                payload = payloads_policy.apply(
                    self.dp,
                    appsensor_meta,
                    type_of_param,
                    vuln_param,
                    vuln_results.get("value"),
                    meta,
                    pattern
                )

                sendEvent(
                    appsensor_meta,
                    self.dp,
                    vuln_param,
                    meta,
                    payload,
                    pattern)

                return True

        return False

    def check_dp_violation(self, param_name, param_value):
        rules = self.get_ruleset()
        if rules:
            return rules.check_violation(param_name, param_value, self.active_pattern_ids, self.v1_compatability_enabled)
        return None

    def get_ruleset(self):
        return rule_manager.get_ruleset_for(self.dp)

    def __str__(self):
        return "<%s enabled: %s dp: %s exclude_headers: %s exclude_forms: %s exclude_cookies: %s v1_compatability_enabled: %s active_pattern_ids: %s exclusions: %s>" % \
            (type(self).__name__, self.enabled, self.dp, self.exclude_headers, self.exclude_forms, self.exclude_cookies, self.v1_compatability_enabled, self.active_pattern_ids, self.exclusions)



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function
from future.standard_library import install_aliases

#install_aliases()
#from urllib.parse import urlparse, urlencode

from tcell_agent.policies import TCellPolicy
from tcell_agent.policies.content_security_policy import ContentSecurityPolicy, ContentSecurityHeader

import logging
logger = logging.getLogger('tcell_agent').getChild(__name__)

from future.backports.urllib.parse import urlparse
from future.backports.urllib.parse import urlencode
from future.backports.urllib.parse import urlsplit
from future.backports.urllib.parse import urlunsplit
from future.backports.urllib.parse import parse_qsl
from future.backports.urllib.parse import urlunparse

class ClickjackingHeader(ContentSecurityHeader):
    def __init__(self, csp_type, value, report_uri=None, policy_id=None):
        self.csp_type = None
        self.value = None
        self.policy_id = None
        if (csp_type is not None):
            if (csp_type.lower() == "content-security-policy"):
                self.csp_type = "csp"
            elif (csp_type == "csp"):
                self.csp_type = csp_type
            else:
                raise Exception("Unknown header")
        self.report_uri = report_uri
        self.value = value


class ClickjackingPolicy(ContentSecurityPolicy):
    def __init__(self, policy_json=None):
        super(ClickjackingPolicy, self).__init__(policy_json, header_class=ClickjackingHeader)



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function

from tcell_agent.agent import TCellAgent
import os
from tcell_agent.sensor_events.httptx import HttpTxSensorEvent, FingerprintSensorEvent, LoginSuccessfulSensorEvent, LoginFailureSensorEvent
from tcell_agent.sensor_events.http_redirect import RedirectSensorEvent
import uuid

from tcell_agent.sanitize import SanitizeUtils
from tcell_agent.instrumentation import handle_exception, safe_wrap_function

from future.backports.urllib.parse import urlsplit
from future.backports.urllib.parse import urlunsplit
from future.backports.urllib.parse import parse_qs
from tcell_agent.appsensor.django import django_request_appsensor, django_response_appsensor, django_path_parameters_appsensor
from django.core.urlresolvers import Resolver404, resolve
from tcell_agent.instrumentation.django.routes import get_route_table
from django.http import HttpResponse

import logging
import tcell_agent.tcell_logger
LOGGER = logging.getLogger('tcell_agent').getChild(__name__)

import django
from tcell_agent.instrumentation.django.utils import django15or16, isDjango15

class TCellLastMiddleware(object):
    _threadmap = {}

    def set_route_id_for_request(self, request):
        route_table = get_route_table()
        try:
            if isDjango15:
                current_info = resolve(request.path_info)
                request._tcell_context.route_id = route_table.get(current_info.func,{}).get("route_id")
            elif django15or16==False and request.resolver_match:
                request._tcell_context.route_id = route_table.get(request.resolver_match.func,{}).get("route_id")
        except Resolver404 as re:
            pass
        except Exception as route_ex:
            LOGGER.debug("Unknown resolver error")
            LOGGER.debug(route_ex)

    def process_request(self, request):
        safe_wrap_function("AppSensor Request", django_request_appsensor, request)

    def process_view(self, request, view_func, view_args, view_kwargs):
        def assign_route_id():
            if request._tcell_context.route_id is None:
                self.set_route_id_for_request(request)
        safe_wrap_function("Assiging route ID to route (view)", assign_route_id)

        if view_kwargs:
            safe_wrap_function("AppSensor Path Parameters", django_path_parameters_appsensor, request, view_kwargs)
        return None

    def process_template_response(self, request, response):
        def assign_route_id():
            if request._tcell_context.route_id is None:
                self.set_route_id_for_request(request)
        safe_wrap_function("Assiging route ID to route (template)", assign_route_id)
        return response

    def process_response(self, request, response):
        def assign_route_id():
            if request._tcell_context.route_id is None:
                self.set_route_id_for_request(request)
        safe_wrap_function("Assiging route ID to route (resp)", assign_route_id)
        safe_wrap_function("AppSensor Response", django_response_appsensor, HttpResponse, request, response)
        return response



from tcell_agent.policies import TCellPolicy

import logging

logger = logging.getLogger('tcell_agent').getChild(__name__)

class PatchesPolicy(TCellPolicy):
    api_identifier = "patches"

    def __init__(self, policy_json=None):
        super(PatchesPolicy, self).__init__()
        self.policy_id = None
        self.version = None
        self.ip_blocking_enabled = False
        self.blocked_ips = {}

        if policy_json is not None:
            self.load_from_json(policy_json)
    def is_ip_blocked(self, ip_address):
        return self.ip_blocking_enabled and self.blocked_ips.get(ip_address, False)

    def load_from_json(self, policy_json):
        if "policy_id" in policy_json:
            self.policy_id = policy_json["policy_id"]
        else:
            raise Exception("Policy Id Not Found")

        if "version" in policy_json:
            self.version = policy_json["version"]

        if 1 is not self.version:
            logger.warn("Patches Policy not supported: %s" % self.version)
            return

        policy_data = policy_json.get("data")

        if policy_data:
            blocked_ips = policy_data.get("blocked_ips")
            if blocked_ips:
                for ip_info in blocked_ips:
                    if "ip" in ip_info:
                        self.ip_blocking_enabled = True
                        self.blocked_ips[ip_info["ip"]] = True




from setuptools import setup, find_packages
from tcell_agent.version import VERSION

setup(name='tcell_agent',
      version=VERSION,
      description='tCell.io Agent',
      url='https://tcell.io',
      author='tCell.io',
      author_email='support@tcell.io',
      license='Free-to-use, proprietary software.',
      install_requires=[
          "requests[security]",
          "future",
          "pyyaml"
      ],
      tests_require=[
          "requests[security]",
          "future",
          "nose",
          "gunicorn",
          "Django",
          "httmock"
      ],
      test_suite = 'nose.collector',
      scripts=['tcell_agent/bin/tcell_agent'],
      packages=find_packages()+['tcell_agent/pythonpath'],
      package_data = { 'tcell_agent.appsensor.rules': ['*.json'] }
)



# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function
from . import SensorEvent
from tcell_agent.sanitize import SanitizeUtils
from future.backports.urllib.parse import urlsplit

"""
"event_type":"login",
"event_name":"login-success",
"user_agent":"Mozilla/5.0 ...",
"referrer":"http://localhost:3085/users/sign_in",
"remote_addr":"10.0.2.2",
"header_keys":["VERSION","HOST","CONNECTION","CACHE_CONTROL","COOKIE"],
"user_id":"1",
"document_uri":"/users/sign_in",
"session":"e9e80cd52ad521ddb9090ac9ac",
"user_valid": true
"""

class LoginEvent(SensorEvent):
    def __init__(self):
        super(LoginEvent, self).__init__("login")
        self.raw_referrer = None
        self.raw_uri = None

    def success(self, *args, **kwargs):
        self["event_name"] = "login-success"
        return self._add_details(*args, **kwargs)

    def failure(self, *args, **kwargs):
        self["event_name"] = "login-failure"
        return self._add_details(*args, **kwargs)

    def _add_details(self,user_id, user_agent, referrer, remote_addr, header_keys, document_uri, session_id=None):
        if user_id:
            self["user_id"] = user_id
        self["user_agent"] = user_agent
        self.raw_referrer = referrer
        self["remote_addr"] = remote_addr
        self["header_keys"] = header_keys
        self.raw_uri = document_uri
        if session_id:
            self["session"] = session_id
        return self

    def post_process(self):
        if self.raw_uri is not None:
            #print(self.raw_uri)
            self["document_uri"] = SanitizeUtils.strip_uri(self.raw_uri)
        if self.raw_referrer is not None:
            #print(self.raw_referrer)
            self["referrer"] = SanitizeUtils.strip_uri(self.raw_referrer)




# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from __future__ import print_function

from django.test import TestCase
from mock import patch
from django.http import HttpResponse
from django.conf.urls import include, url

try:
    import django
    from django.conf import settings
    settings.configure()
    settings.DATABASES = {
    'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'mydatabase',
    }
    }
    django.setup()
except RuntimeError:
    print("Django already setup")

from mock import Mock
from .....policies.dataloss_policy import DataLossPolicy

from .....instrumentation.django.middleware.globalrequestmiddleware import GlobalRequestMiddleware
from .....instrumentation.django.middleware.afterauthmiddleware import AfterAuthMiddleware
from .....instrumentation.django.middleware.body_filter_middleware import BodyFilterMiddleware
from .....instrumentation.django.middleware.tcell_data_exposure_middleware import TCellDataExposureMiddleware
from .....instrumentation.django.middleware.tcelllastmiddleware import TCellLastMiddleware

from django.middleware.common import CommonMiddleware
from .....config import CONFIGURATION
from .....agent import TCellAgent, PolicyTypes

from httmock import urlmatch, HTTMock
import requests
import json
from django.conf import settings

from django.test.client import RequestFactory

try:
    settings.configure()
except:
    pass
settings.DEFAULT_CHARSET = 'utf-8'

import logging
class MockLoggingHandler(logging.Handler):
    """Mock logging handler to check for expected logs.

    Messages are available from an instance's ``messages`` dict, in order, indexed by
    a lowercase log level string (e.g., 'debug', 'info', etc.).
    """

    def __init__(self, *args, **kwargs):
        self.messages = {'debug': [], 'info': [], 'warning': [], 'error': [],
                         'critical': []}
        super(MockLoggingHandler, self).__init__(*args, **kwargs)

    def emit(self, record):
        "Store a message from ``record`` in the instance's ``messages`` dict."
        self.acquire()
        try:
            self.messages[record.levelname.lower()].append(record.getMessage())
        finally:
            self.release()

    def reset(self):
        self.acquire()
        try:
            for message_list in self.messages.values():
                message_list.clear()
        finally:
            self.release()

class DLPMiddlewareTest(TestCase):

    def setUp(self):
        settings.ALLOWED_HOSTS = ['testserver','test.tcell.io']
        settings.ROOT_URLCONF = 'tcell_agent.tests.instrumentation.django.middleware.test_urls'

        self.grm = GlobalRequestMiddleware()
        self.aam = AfterAuthMiddleware()
        self.cm = CommonMiddleware()
        self.bfm = BodyFilterMiddleware()
        self.data_exposure_middleware = TCellDataExposureMiddleware()
        self.tcell_last_middleware = TCellLastMiddleware()

        self.rf = RequestFactory()

        CONFIGURATION.version = 1
        CONFIGURATION.api_key = "Test_-ApiKey=="
        CONFIGURATION.app_id = "TestAppId-AppId"
        CONFIGURATION.host_identifier = "TestHostIdentifier"
        CONFIGURATION.uuid = "test-uuid-test-uuid"
        CONFIGURATION.fetch_policies_from_tcell = False

        added_events = []
        self.added_events = added_events
        def addEvent(self, event):
          event.post_process()
          added_events.append(event)
 
        self.old_add_events = TCellAgent.addEvent
        TCellAgent.addEvent = addEvent

        self.old_tcell_agent = TCellAgent.tCell_agent
        TCellAgent.tCell_agent = TCellAgent()
        TCellAgent.tCell_agent.policies = {}

        self.mock_logger = logging.getLogger("mockery")
        for h in list(self.mock_logger.handlers):
          self.mock_logger.removeHandler(h)
        mock_handler = MockLoggingHandler(level='DEBUG')
        self.mock_logger.addHandler(mock_handler)
        self.mock_logger_messages = mock_handler.messages

    def request_with_session(self, 
        url='http://test.tcell.io/hello/', 
        session_key="101012301200123",
        post_form_data=None):
        if post_form_data is not None:
            request = self.rf.post(url, post_form_data)
        else:
            request = self.rf.get(url)
        request.session = Mock()
        request.session.session_key = session_key
        return request

    def middleware_before(self, request, route_id="mock_request_id"):
        self.grm.process_request(request)
        request._tcell_context.route_id = route_id
        self.data_exposure_middleware.process_request(request)
        self.aam.process_request(request)
        self.cm.process_request(request)
        self.bfm.process_request(request)
        self.tcell_last_middleware.process_request(request)

    def middleware_after(self, request, response):
        self.tcell_last_middleware.process_response(request, response)
        self.bfm.process_response(request, response)
        self.cm.process_response(request, response)
        self.aam.process_response(request, response)
        self.data_exposure_middleware.process_response(request, response)
        self.grm.process_response(request, response)

    def test_sesssion_id_protect(self):

        request = self.request_with_session()
        policy_json = {
            "dlp":{
              "policy_id":"nyzd", 
              "data":{
                "session_id_protections":{"body":["redact"], "log":["event"]}
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_json, cache=False)
        self.middleware_before(request)
        response =  HttpResponse("<html>\n<head>Title</head><body>hello world: 101012301200123 101012301200123</body><html>",content_type="application/html")
        self.middleware_after(request, response)

        expected = b"<html>\n<head>Title</head><body>hello world: [redact] [redact]</body><html>"
        self.assertEqual(response.content, expected)

    def test_db_protect(self):
        request = self.request_with_session()
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "db_protections":[
                  {"scope":"route",
                   "route_ids":["abcdefg"],
                   "tables":["work_infos"],
                   "fields":["SSN"],
                   "actions":{"body":["redact"]}
                  },
                  {"id":6,
                   "tables":["work_infos"],
                   "fields":["income"],
                   "actions":{"body":["redact"], "log":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)
        self.middleware_before(request)
        actions = TCellAgent.get_policy("dlp").get_actions_for_db_field("test","test","work_infos","income")
        for action in actions:
            request._tcell_context.add_response_db_filter("hello", action, "test", "test", "work_infos", "income")
        response =  HttpResponse("<html>\n<head>Title</head><body>hello world: 101012301200123 \n\nhello 101012301200123</body><html>",content_type="application/html")
        self.middleware_after(request, response)

        expected = b"<html>\n<head>Title</head><body>[redact] world: 101012301200123 \n\n[redact] 101012301200123</body><html>"
        self.assertEqual(response.content, expected)

        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], 6)
        self.assertEqual(self.added_events[0]["table"], "work_infos")

    def test_db_protect_by_route(self):
        request = self.request_with_session()
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "db_protections":[
                  {"scope":"route",
                   "route_ids":["abcdefg"],
                   "tables":["work_infos"],
                   "fields":["SSN"],
                   "actions":{"body":["redact"]}
                  },
                  {"id":6,
                   "tables":["work_infos"],
                   "fields":["income"],
                   "actions":{"body":["redact"], "log":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)

        self.middleware_before(request, route_id="abcdefg")
        actions = TCellAgent.get_policy("dlp").get_actions_for_db_field("test","test","work_infos","income",route="abcdefg")
        for action in actions:
            request._tcell_context.add_response_db_filter("hello", action, "test", "test", "work_infos", "SSN")
        response =  HttpResponse("<html>\n<head>Title</head><body>hello world: 101012301200123 \n\nhello 101012301200123</body><html>",content_type="application/html")
        self.middleware_after(request, response)

        expected = b"<html>\n<head>Title</head><body>[redact] world: 101012301200123 \n\n[redact] 101012301200123</body><html>"
        self.assertEqual(response.content, expected)

        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], 6)
        self.assertEqual(self.added_events[0]["table"], "work_infos")

    def test_request_protect_by_route(self):
        request = self.request_with_session(url='http://test.tcell.io/hello/?password=hello')
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "request_protections":[
                  {"id":6,
                   "scope":"route",
                   "route_ids":["abcdef"],
                   "variable_context":"form",
                   "variables":["password"],
                   "actions":{"body":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)
        self.middleware_before(request, route_id="abcdef")
        response =  HttpResponse("<html>\n<head>Title</head><body>hello world: 101012301200123 \n\nhello 101012301200123</body><html>",content_type="application/html")
        self.middleware_after(request, response)

        expected = b"<html>\n<head>Title</head><body>[redact] world: 101012301200123 \n\n[redact] 101012301200123</body><html>"
        self.assertEqual(response.content, expected)

        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], 6)

    def test_request_protect_by_route_post(self):
        request = self.request_with_session(
          url='http://test.tcell.io/hello/login',
          post_form_data={"a":"b","password":"hello world"})
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "request_protections":[
                  {"id":22,
                   "scope":"route",
                   "route_ids":["abcdef"],
                   "variable_context":"form",
                   "variables":["password"],
                   "actions":{"body":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)
        self.middleware_before(request, route_id="abcdef")
        response =  HttpResponse("<html>\n<head>Title</head><body>hello world: 101012301200123 \n\nhello 101012301200123</body><html>",content_type="application/html")
        self.middleware_after(request, response)

        expected = b"<html>\n<head>Title</head><body>[redact]: 101012301200123 \n\nhello 101012301200123</body><html>"
        self.assertEqual(response.content, expected)

        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], 22)

    def test_request_cookie_protect_by_route(self):
        request = self.request_with_session(url='http://test.tcell.io/hello/?password=sammy')
        request.COOKIES["MyCookie"] = "hello"
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "request_protections":[
                  {"id":6,
                   "scope":"route",
                   "route_ids":["abcdef"],
                   "variable_context":"cookie",
                   "variables":["MyCookie"],
                   "actions":{"body":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)

        self.middleware_before(request, route_id="abcdef")
        response =  HttpResponse("<html>\n<head>Title</head><body>hello world: 101012301200123 \n\nhello 101012301200123</body><html>",content_type="application/html")
        self.middleware_after(request, response)

        expected = b"<html>\n<head>Title</head><body>[redact] world: 101012301200123 \n\n[redact] 101012301200123</body><html>"
        self.assertEqual(response.content, expected)

        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], 6)

    def test_request_header_protect_by_route(self):
        request = self.request_with_session(url='http://test.tcell.io/hello/?password=sammy')
        request.META["HTTP_MY_HEADER"] = "heyathere"
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "request_protections":[
                  {"id":"100",
                   "scope":"route",
                   "route_ids":["abcdef"],
                   "variable_context":"header",
                   "variables":["My_Header"],
                   "actions":{"body":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)

        self.middleware_before(request, route_id="abcdef")
        response =  HttpResponse("<html>\n<head>Title</head><body>heyathere world: 101012301200123 \n\nheyathere hello 101012301200123</body><html>",content_type="application/html")
        self.middleware_after(request, response)

        expected = b"<html>\n<head>Title</head><body>[redact] world: 101012301200123 \n\n[redact] hello 101012301200123</body><html>"
        self.assertEqual(response.content, expected)

        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], "100")


    def test_logger_header_protect_by_route(self):
        request = self.request_with_session(url='http://test.tcell.io/hello/?password=sammy')
        request.META["HTTP_MY_HEADER"] = "heyathere"
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "request_protections":[
                  {"id":"100",
                   "scope":"route",
                   "route_ids":["abcdef"],
                   "variable_context":"header",
                   "variables":["My_Header"],
                   "actions":{"body":["redact"],"log":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)
        self.middleware_before(request, route_id="abcdef")

        from tcell_agent.instrumentation.django import dlp
        dlp.dlp_instrumentation()

        self.mock_logger.warn("Hi heyathere There")

        self.assertEqual(self.mock_logger_messages["warning"][0], "Hi [redact] There")
        
        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], "100")

    def test_logger_header_protect_global(self):
        request = self.request_with_session(url='http://test.tcell.io/hello/?password=sammy')
        request.META["HTTP_MY_HEADER"] = "heyathere"
        policy_db_protection_by_route = {
            "dlp":{
              "v":1,
              "policy_id":"xyz-def",
              "data":{
                "request_protections":[
                  {"id":"100",
                   "variable_context":"header",
                   "variables":["My_Header"],
                   "actions":{"body":["redact"],"log":["redact"]}
                  }
                ]
              }
            }
        }
        TCellAgent.tCell_agent.process_policies(policy_db_protection_by_route, cache=False)
        self.middleware_before(request, route_id=None)

        from tcell_agent.instrumentation.django import dlp
        dlp.dlp_instrumentation()

        self.mock_logger.warn("Hi heyathere There")
        self.assertEqual(self.mock_logger_messages["warning"][0], "Hi [redact] There")
        
        self.assertEqual(len(self.added_events), 1)
        self.assertEqual(self.added_events[0]["event_type"], "dlp")
        self.assertEqual(self.added_events[0]["rule"], "100")



from __future__ import unicode_literals
from __future__ import print_function

import json
import unittest

from mock import patch
from tcell_agent.appsensor.meta import AppSensorMeta
from tcell_agent.policies.appsensor import InjectionSensor
from tcell_agent.policies.appsensor import LoginSensor
from tcell_agent.policies.appsensor import MiscSensor
from tcell_agent.policies.appsensor import RequestSizeSensor
from tcell_agent.policies.appsensor import ResponseCodesSensor
from tcell_agent.policies.appsensor import ResponseSizeSensor
from tcell_agent.policies.appsensor import UserAgentSensor
from tcell_agent.policies.appsensor_policy import AppSensorPolicy

# set is builtin in python 3 but a module in earlier version
try:
    set
except NameError:
    from sets import Set as set

policy_one_disabled = """
{
    "policy_id":"abc-abc-abc",
    "data": {
    }
}
"""

policy_one = """
{
    "policy_id":"abc-abc-abc",
    "data": {
        "options": {
            "xss":true
        }
    }
}
"""

policy_v2_req_size = """
{
    "policy_id":"abc-abc-abc",
    "version":2,
    "data": {
        "options": {
            "payloads": {}
        },
        "sensors": {
            "req_size": {
                "limit":1024,
                "exclude_routes":["2300"]
            }
        }
    }
}
"""

policy_v2_all_options = """
{
    "policy_id":"abc-abc-abc",
    "version":2,
    "data": {
        "options": {
            "payloads": {
                "send_payloads": true,
                "send_blacklist": {
                    "JSESSIONID": ["cookie"],
                    "ssn": ["*"],
                    "password": ["*"]
                },
                "send_whitelist": {},
                "log_payloads": true,
                "log_blacklist": {},
                "log_whitelist": {
                    "username": ["*"]
                }
            }
        },
        "sensors": {
            "req_size": {
                "limit":1024,
                "exclude_routes":["2300"]
            },
            "resp_size": {
                "limit":2048,
                "exclude_routes":["2323"]
            },
            "resp_codes": {
                "series_400_enabled":true,
                "series_500_enabled":true
            },
            "xss": {
                "libinjection":true,
                "patterns":["1","2","8"],
                "exclusions":{
                    "bob":["*"]
                }
            },
            "sqli":{
                "libinjection":true,
                "exclude_headers":true,
                "patterns":["1"]
            },
            "fpt":{
                "patterns":["1","2"],
                "exclude_forms":true,
                "exclude_cookies":true,
                "exclusions":{
                    "somethingcommon":["form"]
                }
            },
            "cmdi":{
                 "patterns":["1","2"]
            },
            "nullbyte":{
                 "patterns":["1","2"]
            },
            "retr":{
                "patterns":["1","2"]
            },
            "ua": {
                "empty_enabled": true
            },
            "login":{
                "lgnSccss_enabled":true,
                "lgnFlr_enabled":true,
                "psswdRstReq":true,
                "psswdRstAttmpt":true,
                "psswdRst":true
            },
            "errors":{
                "csrf_exception_enabled": true,
                "sql_exception_enabled": true
            },
            "database":{
                "large_result": {
                    "limit": 10
                }
            }
        }
    }
}
"""

class AppSensorPolicyTest(unittest.TestCase):
    def classname_test(self):
        self.assertEqual(AppSensorPolicy.api_identifier, "appsensor")

    def read_appensor_v1_policy_disabled_test(self):
        policy_json = json.loads(policy_one_disabled)
        policy = AppSensorPolicy()
        policy.loadFromJson(policy_json)

        self.assertIsNone(policy.options.get("req_size"))
        self.assertIsNone(policy.options.get("resp_size"))
        self.assertIsNone(policy.options.get("resp_codes"))
        self.assertIsNone(policy.options.get("xss"))
        self.assertIsNone(policy.options.get("sqli"))
        self.assertIsNone(policy.options.get("cmdi"))
        self.assertIsNone(policy.options.get("fpt"))
        self.assertIsNone(policy.options.get("nullbyte"))
        self.assertIsNone(policy.options.get("retr"))
        self.assertIsNone(policy.options.get("login"))
        self.assertIsNone(policy.options.get("ua"))
        self.assertIsNone(policy.options.get("errors"))
        self.assertIsNone(policy.options.get("database"))


    def read_appensor_v1_policy_test(self):
        policy_json = json.loads(policy_one)
        policy = AppSensorPolicy()
        policy.loadFromJson(policy_json)

        self.assertIsNotNone(policy.payloads_policy)
        self.assertIsNotNone(policy.options["req_size"])
        self.assertIsNotNone(policy.options["resp_size"])
        self.assertIsNotNone(policy.options["resp_codes"])
        self.assertIsNotNone(policy.options["xss"])
        self.assertIsNotNone(policy.options["sqli"])
        self.assertIsNotNone(policy.options["cmdi"])
        self.assertIsNotNone(policy.options["fpt"])
        self.assertIsNotNone(policy.options["nullbyte"])
        self.assertIsNotNone(policy.options["retr"])
        self.assertIsNotNone(policy.options["login"])
        self.assertIsNotNone(policy.options["ua"])
        self.assertIsNotNone(policy.options["errors"])
        self.assertIsNotNone(policy.options["database"])

        self.assertFalse(policy.options["req_size"].enabled)
        self.assertFalse(policy.options["resp_size"].enabled)
        self.assertFalse(policy.options["resp_codes"].enabled)
        self.assertTrue(policy.options["xss"].enabled)
        self.assertFalse(policy.options["cmdi"].enabled)
        self.assertFalse(policy.options["fpt"].enabled)
        self.assertFalse(policy.options["nullbyte"].enabled)
        self.assertFalse(policy.options["retr"].enabled)
        self.assertFalse(policy.options["login"].enabled)
        self.assertFalse(policy.options["ua"].enabled)
        self.assertFalse(policy.options["errors"].csrf_exception_enabled)
        self.assertFalse(policy.options["errors"].sql_exception_enabled)
        self.assertFalse(policy.options["database"].enabled)

        self.assertTrue(policy.options["resp_codes"].series_400_enabled)
        self.assertTrue(policy.options["resp_codes"].series_500_enabled)

        self.assertTrue(policy.options["xss"].v1_compatability_enabled)
        self.assertTrue(policy.options["sqli"].v1_compatability_enabled)
        self.assertTrue(policy.options["cmdi"].v1_compatability_enabled)
        self.assertTrue(policy.options["fpt"].v1_compatability_enabled)
        self.assertTrue(policy.options["nullbyte"].v1_compatability_enabled)
        self.assertTrue(policy.options["retr"].v1_compatability_enabled)

        self.assertTrue(policy.payloads_policy.send_payloads)
        self.assertTrue(policy.payloads_policy.log_payloads)
        self.assertEqual(policy.payloads_policy.send_blacklist, policy.payloads_policy.DEFAULT_BLACKLIST)
        self.assertEqual(policy.payloads_policy.send_whitelist, {})
        self.assertFalse(policy.payloads_policy.use_send_whitelist)
        self.assertEqual(policy.payloads_policy.log_blacklist, policy.payloads_policy.DEFAULT_BLACKLIST)
        self.assertEqual(policy.payloads_policy.log_whitelist, {})
        self.assertFalse(policy.payloads_policy.use_log_whitelist)

    def read_appensor_v2_req_size_policy_test(self):
        policy_json = json.loads(policy_v2_req_size)
        policy = AppSensorPolicy()
        policy.loadFromJson(policy_json)

        self.assertIsNotNone(policy.payloads_policy)
        self.assertIsNotNone(policy.options["req_size"])
        self.assertIsNotNone(policy.options["resp_size"])
        self.assertIsNotNone(policy.options["resp_codes"])
        self.assertIsNotNone(policy.options["xss"])
        self.assertIsNotNone(policy.options["sqli"])
        self.assertIsNotNone(policy.options["cmdi"])
        self.assertIsNotNone(policy.options["fpt"])
        self.assertIsNotNone(policy.options["nullbyte"])
        self.assertIsNotNone(policy.options["retr"])
        self.assertIsNotNone(policy.options["login"])
        self.assertIsNotNone(policy.options["ua"])
        self.assertIsNotNone(policy.options["errors"])
        self.assertIsNotNone(policy.options["database"])

        self.assertTrue(policy.options["req_size"].enabled)
        self.assertFalse(policy.options["resp_size"].enabled)
        self.assertFalse(policy.options["resp_codes"].enabled)
        self.assertFalse(policy.options["xss"].enabled)
        self.assertFalse(policy.options["cmdi"].enabled)
        self.assertFalse(policy.options["fpt"].enabled)
        self.assertFalse(policy.options["nullbyte"].enabled)
        self.assertFalse(policy.options["retr"].enabled)
        self.assertFalse(policy.options["login"].enabled)
        self.assertFalse(policy.options["ua"].enabled)
        self.assertFalse(policy.options["errors"].csrf_exception_enabled)
        self.assertFalse(policy.options["errors"].sql_exception_enabled)
        self.assertFalse(policy.options["database"].enabled)

        self.assertFalse(policy.payloads_policy.send_payloads)
        self.assertFalse(policy.payloads_policy.log_payloads)
        self.assertEqual(policy.payloads_policy.send_blacklist, policy.payloads_policy.DEFAULT_BLACKLIST)
        self.assertEqual(policy.payloads_policy.send_whitelist, {})
        self.assertFalse(policy.payloads_policy.use_send_whitelist)
        self.assertEqual(policy.payloads_policy.log_blacklist, policy.payloads_policy.DEFAULT_BLACKLIST)
        self.assertEqual(policy.payloads_policy.log_whitelist, {})
        self.assertFalse(policy.payloads_policy.use_log_whitelist)

    def read_appensor_v2_all_options_policy_test(self):
        policy_json = json.loads(policy_v2_all_options)
        policy = AppSensorPolicy()

        policy.loadFromJson(policy_json)

        self.assertIsNotNone(policy.payloads_policy)
        self.assertIsNotNone(policy.options["req_size"])
        self.assertIsNotNone(policy.options["resp_size"])
        self.assertIsNotNone(policy.options["resp_codes"])
        self.assertIsNotNone(policy.options["xss"])
        self.assertIsNotNone(policy.options["sqli"])
        self.assertIsNotNone(policy.options["cmdi"])
        self.assertIsNotNone(policy.options["fpt"])
        self.assertIsNotNone(policy.options["nullbyte"])
        self.assertIsNotNone(policy.options["retr"])
        self.assertIsNotNone(policy.options["login"])
        self.assertIsNotNone(policy.options["ua"])
        self.assertIsNotNone(policy.options["errors"])
        self.assertIsNotNone(policy.options["database"])

        self.assertTrue(policy.options["req_size"].enabled)
        self.assertTrue(policy.options["resp_size"].enabled)
        self.assertTrue(policy.options["resp_codes"].enabled)
        self.assertTrue(policy.options["xss"].enabled)
        self.assertTrue(policy.options["cmdi"].enabled)
        self.assertTrue(policy.options["fpt"].enabled)
        self.assertTrue(policy.options["nullbyte"].enabled)
        self.assertTrue(policy.options["retr"].enabled)
        self.assertTrue(policy.options["login"].enabled)
        self.assertTrue(policy.options["ua"].enabled)
        self.assertTrue(policy.options["ua"].empty_enabled)
        self.assertTrue(policy.options["errors"].csrf_exception_enabled)
        self.assertTrue(policy.options["errors"].sql_exception_enabled)
        self.assertTrue(policy.options["database"].enabled)
        self.assertEqual(policy.options["database"].max_rows, 10)

        self.assertTrue(policy.payloads_policy.send_payloads)
        self.assertTrue(policy.payloads_policy.log_payloads)
        self.assertEqual(policy.payloads_policy.send_blacklist, {
            "jsessionid": set(["cookie"]),
            "ssn": set(["*"]),
            "password": set(["*"])
        })
        self.assertEqual(policy.payloads_policy.send_whitelist, {})
        self.assertTrue(policy.payloads_policy.use_send_whitelist)
        self.assertEqual(policy.payloads_policy.log_blacklist, {})
        self.assertEqual(policy.payloads_policy.log_whitelist, {
            "username": set(["*"])
        })
        self.assertTrue(policy.payloads_policy.use_log_whitelist)

    def test_run_for_response(self):
        policy_json = json.loads(policy_v2_all_options)
        policy = AppSensorPolicy()
        policy.loadFromJson(policy_json)

        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        meta.response_data(1024, 200)

        with patch.object(ResponseSizeSensor, 'check') as patched_size_check:
            with patch.object(ResponseCodesSensor, 'check') as patched_codes_check:
                policy.run_for_response(meta)
                patched_size_check.assert_called_once_with(meta, 1024)
                patched_codes_check.assert_called_once_with(meta, 200)

    def test_check_login_failure(self):
        policy_json = json.loads(policy_v2_all_options)
        policy = AppSensorPolicy()
        policy.loadFromJson(policy_json)

        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        with patch.object(LoginSensor, 'check') as patched_check:
            policy.check_login_failure(meta, "username")
            patched_check.assert_called_once_with(meta, "username")

    def test_run_for_request_with_no_params(self):
        policy_json = json.loads(policy_v2_all_options)
        policy = AppSensorPolicy()
        policy.loadFromJson(policy_json)

        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        meta.request_data(1024, {}, {}, {}, "", None, {})

        with patch.object(RequestSizeSensor, 'check') as patched_check:
            with patch.object(InjectionSensor, 'check') as patched_injection_check:
                with patch.object(UserAgentSensor, 'check') as patched_ua_check:
                    policy.run_for_request(meta, {})
                    patched_check.assert_called_once_with(meta, 1024)
                    patched_ua_check.assert_called_once_with(meta)
                    self.assertFalse(patched_injection_check.called)

    def test_csrf_rejected(self):
        policy_json = json.loads(policy_v2_all_options)
        policy = AppSensorPolicy()
        policy.loadFromJson(policy_json)

        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        with patch.object(MiscSensor, 'csrf_rejected') as patched_csrf_rejected:
            policy.csrf_rejected(meta, "Missing CSRF")
            patched_csrf_rejected.assert_called_once_with(meta, "Missing CSRF")



from __future__ import unicode_literals
from __future__ import print_function

import unittest

from ...config import CONFIGURATION
from ...api import TCellAPI
from ...sensor_events import SensorEvent

from future.backports.urllib.parse import urlparse
from future.backports.urllib.parse import parse_qs

from httmock import urlmatch, HTTMock
import requests
import json

class CustomEvent(SensorEvent):
  def __init__(self):
    super(CustomEvent, self).__init__("chegg_custom_event")
    self["stuff"] = {
      "test2":["a","b","c"]
    }

class SDKTest(unittest.TestCase):

    def get_mock(self):
      @urlmatch(netloc=r'input\.tcell\.io$', path=r'^/api/v1/app/TestAppId-AppId/chegg_endpoint$')
      def tcell_mock(url, request):
        self.assertEqual(request.headers["Content-type"], "application/json")
        if request.headers["Authorization"] != "Bearer Test_-ApiKey==":
          return {'status_code':403}
        body_json = json.loads(request.body)
        events_json = body_json.get("chegg_events")
        if (events_json is None):
          return {'status_code': 300}
        if (events_json[0].get("stuff").get("test2") == ["a","b","c"] and
            events_json[0].get("event_type") == "chegg_custom_event" and
            body_json.get("hostname") == "TestHostIdentifier" and
            body_json.get("uuid") == "test-uuid-test-uuid"):
          return {'status_code': 200}
        return {'status_code': 404}
      return tcell_mock

    def chegg_test_ok(self):

        CONFIGURATION.version = 1
        CONFIGURATION.api_key = "Test_-ApiKey=="
        CONFIGURATION.app_id = "TestAppId-AppId"
        CONFIGURATION.host_identifier = "TestHostIdentifier"
        CONFIGURATION.uuid = "test-uuid-test-uuid"


        with HTTMock(self.get_mock()):
          c = CustomEvent()
          c["oh"] = {"more":"stuff"}
          response = TCellAPI.send_events([c], event_wrap_name="chegg_events", event_endpoint="chegg_endpoint")
          self.assertEqual(response.status_code, 200)

    def chegg_test_bad(self):
        CONFIGURATION.version = 1
        CONFIGURATION.api_key = "Test_-ApiKey=="
        CONFIGURATION.app_id = "TestAppId-AppId"
        CONFIGURATION.host_identifier = "TestHostIdentifier"
        CONFIGURATION.uuid = "test-uuid-test-uuid"

        with HTTMock(self.get_mock()):
          c = CustomEvent()
          response = TCellAPI.send_events([c], event_wrap_name="bad_wrapper", event_endpoint="chegg_endpoint")
          self.assertEqual(response.status_code, 300)




from __future__ import unicode_literals
from __future__ import print_function

import unittest
from ...policies.honeytoken_policy import HoneytokenPolicy
import json

from future.backports.urllib.parse import urlparse
from future.backports.urllib.parse import parse_qs

policy_one = """
{
  "v":1,
  "policy_id":"xyz-def",
  "data": {
    "token_salt":"abcddef",
    "credentials":[
        {"id":"myid1", "token":"jjtokenjj"}
    ]
  }
}
"""

policy_nosalt = """
{
  "v":1,
  "policy_id":"xyz-def",
  "data": {
    "credentials":[
        {"id":"myid1", "token":"jjtokenjj"}
    ]
  }
}
"""

policy_empty = """
{
  "v":1,
  "policy_id":"xyz-def",
  "data": {
  }
}
"""

class HoneytokenPolicyTest(unittest.TestCase):

    def create_token_test(self):
        policy = HoneytokenPolicy()
        token = policy.create_credential_token("admin@tcell.io","admin","testsalt")
        self.assertEqual(token, "5492df8eb107b81b24b91d84d72512a3c765cc013d2b7af7d5f9fa0832811624")

    def empty_policy_test(self):
        policy_json = json.loads(policy_empty)
        policy = HoneytokenPolicy()

        self.assertEqual(policy.token_salt, None)
        self.assertEqual(policy.cred_tokens, None)

        policy.loadFromJson(policy_json)

        self.assertEqual(policy.token_salt, None)
        self.assertEqual(policy.cred_tokens, None)

    def no_salt_policy_test(self):
        policy_json = json.loads(policy_nosalt)
        policy = HoneytokenPolicy()

        self.assertEqual(policy.token_salt, None)
        self.assertEqual(policy.cred_tokens, None)

        policy.loadFromJson(policy_json)

        self.assertEqual(policy.token_salt, None)
        self.assertEqual(policy.cred_tokens, None)

    def normal_policy_test(self):
        policy_json = json.loads(policy_one)
        policy = HoneytokenPolicy()
        policy.loadFromJson(policy_json)

        self.assertEqual(policy.token_salt, "abcddef")
        self.assertEqual(policy.cred_tokens, {"jjtokenjj":"myid1"})


from __future__ import unicode_literals
from __future__ import print_function

import unittest
from ...policies.clickjacking_policy import ClickjackingPolicy

from future.backports.urllib.parse import urlparse
from future.backports.urllib.parse import parse_qs

class ContentSecurityPolicyTest(unittest.TestCase):
    def old_header_test(self):
        policy = ClickjackingPolicy()
        policy.loadFromHeaderString("Content-Security-Policy-Report-Only: test123")
        self.assertEqual(policy.headers(), [])

    def new_header_test(self):
        policy_json = {"policy_id":"xyzd", "headers":[{"name":"content-Security-POLICY", "value":"test321"}]}
        policy = ClickjackingPolicy()
        policy.loadFromJson(policy_json)
        self.assertEqual(policy.headers(), [['Content-Security-Policy', 'test321']])

    def header_with_report_uri_test(self):
        policy_json = {"policy_id":"xyzd", 
                       "headers":[{"name":"content-Security-policy", "value":"normalvalue","report-uri":"https://www.example.com/xys"}]
                      }
        policy = ClickjackingPolicy()
        policy.loadFromJson(policy_json)
        policy_headers = policy.headers("1","2","3")
        policy_header = policy_headers[0]
        print(policy_header)
        self.assertEqual(policy_header[0],"Content-Security-Policy")
        policy_header_value_parts = urlparse(policy_header[1].split(" ")[2])
        print(policy_header_value_parts)
        self.assertEqual(policy_header_value_parts.path, "/xys")
        print(policy_header_value_parts.query)
        query_params = parse_qs(policy_header_value_parts.query)
        print(query_params)
        self.assertEqual(query_params["tid"], ["1"])
        self.assertEqual(query_params["rid"], ["2"])
        self.assertEqual(query_params["sid"], ["3"])


from tcell_agent.policies.appsensor.sensor import sendEvent

class UserAgentSensor(object):
    DP_CODE = "uaempty"

    def __init__(self, policy_json=None):
        self.enabled = False
        self.empty_enabled = False
        self.excluded_route_ids = {}

        if policy_json is not None:
            self.enabled = policy_json.get("enabled", False)
            self.empty_enabled = policy_json.get("empty_enabled", False)

            for route_id in policy_json.get("exclude_routes", []):
                self.excluded_route_ids[route_id] = True

    def check(self, appsensor_meta):
        if not self.enabled or not self.empty_enabled:
            return

        if self.excluded_route_ids.get(appsensor_meta.route_id, False):
            return False

        if not(appsensor_meta.user_agent_str and appsensor_meta.user_agent_str.strip()):
            sendEvent(
              appsensor_meta,
              self.DP_CODE,
              None,
              None)

    def __str__(self):
        return "<%s enabled: %s empty_enabled: %s dp_code: %s>" % \
            (type(self).__name__, self.enabled, self.empty_enabled, self.DP_CODE)



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function

from tcell_agent.agent import TCellAgent
from django.conf import settings
from tcell_agent.sensor_events import AppConfigSettings
import json
from .. import safe_wrap_function

def send_django_setting_events():
    safe_wrap_function(
      "Settings: Check cookie sesion length",
      lambda: TCellAgent.send(AppConfigSettings(
        "Django",
        "Session",
        "SESSION_COOKIE_HTTPONLY",
        settings.SESSION_COOKIE_HTTPONLY
      )) 
    )
    safe_wrap_function(
      "Settings: Check cookie sesion secure",
      lambda: TCellAgent.send(AppConfigSettings(
        "Django",
        "Session",
        "SESSION_COOKIE_SECURE",
        settings.SESSION_COOKIE_SECURE
      ))
    ) 
    safe_wrap_function(
      "Settings: Check cookie sesion expiration",
      lambda: TCellAgent.send(AppConfigSettings(
        "Django",
        "Session",
        "SESSION_EXPIRE_AT_BROWSER_CLOSE",
        settings.SESSION_EXPIRE_AT_BROWSER_CLOSE
      )) 
    )
    safe_wrap_function(
      "Settings: Debug turned off",
      lambda: TCellAgent.send(AppConfigSettings(
        "Django",
        "Core",
        "DEBUG",
        settings.DEBUG
      ))
    )
    safe_wrap_function(
      "Settings: Allowed Host Length",
      lambda: TCellAgent.send(AppConfigSettings(
          "Django",
          "Core",
          "ALLOWED_HOSTS_LEN",
          len(settings.ALLOWED_HOSTS)
        ))
    )
    safe_wrap_function(
      "Settings: List of password hashes",
      lambda: TCellAgent.send(AppConfigSettings(
        "Django",
        "Auth",
        "PASSWORD_HASHERS",
        json.dumps(settings.PASSWORD_HASHERS)
      )) 
    )
    safe_wrap_function(
      "Settings: Password reset timeout in days",
      lambda: TCellAgent.send(AppConfigSettings(
        "Django",
        "Auth",
        "PASSWORD_RESET_TIMEOUT_DAYS",
        settings.PASSWORD_RESET_TIMEOUT_DAYS
      )) 
    )


# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function


import uuid
import re

from future.backports.urllib.parse import urlsplit
from future.backports.urllib.parse import urlunsplit
from future.backports.urllib.parse import parse_qs

from tcell_agent.policies import TCellPolicy
from tcell_agent.sensor_events.http_redirect import RedirectSensorEvent

import logging
logger = logging.getLogger('tcell_agent').getChild(__name__)

import hmac
import hashlib
import sys

class HoneytokenPolicy(TCellPolicy):
    api_identifier = "exp-honeytokens"

    def __init__(self, policy_json=None):
        super(HoneytokenPolicy, self).__init__()
        self.cred_tokens = None
        self.token_salt = None
        if policy_json is not None:
            self.loadFromJson(policy_json)

    def get_id_for_credential(self, username, password):
        if self.cred_tokens is None:
            return None
        token = self.create_credential_token(username, password)
        if token is not None:
            return self.cred_tokens.get(token)
        return None

    def create_credential_token(self, username, password, token_salt=None):
        if username is None or password is None:
            return None
        if token_salt is None:
            token_salt = self.token_salt
        if token_salt is None: # still? well we can't hmac that
            return None
        try:
            username_enc = bytes(username, 'utf-8')
            token_salt_enc = bytes(token_salt, 'utf-8')
        except:
            username_enc = bytes(username)
            token_salt_enc = bytes(token_salt)
        passwordhmac = hmac.new(username_enc, str(password).encode('utf-8'), hashlib.sha512).hexdigest()
        combo = username + passwordhmac[:62]
        token = hmac.new(token_salt_enc, str(combo).encode('utf-8'), hashlib.sha256).hexdigest()
        return token
        
    def loadFromJson(self, policy_json):
        if "policy_id" in policy_json:
            self.policy_id = policy_json["policy_id"]
        else:
            raise Exception("Policy Id Not Found")

        policy_data_json = policy_json.get("data")
        if policy_data_json:
            token_salt = policy_data_json.get("token_salt")
            if token_salt is not None:
                self.token_salt = token_salt
                credentials = policy_data_json.get("credentials")
                if credentials is not None:
                    self.cred_tokens = {}
                    for credential in credentials:
                        if credential.get("id") and credential.get("token"):
                            self.cred_tokens[credential.get("token")] = credential.get("id")


# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function

from tcell_agent.agent import TCellAgent
import os
from tcell_agent.sensor_events.httptx import HttpTxSensorEvent, FingerprintSensorEvent, LoginSuccessfulSensorEvent, LoginFailureSensorEvent
from tcell_agent.sensor_events.http_redirect import RedirectSensorEvent
import uuid

from tcell_agent.sanitize import SanitizeUtils
from tcell_agent.instrumentation import handle_exception, safe_wrap_function

from future.backports.urllib.parse import urlsplit
from future.backports.urllib.parse import urlunsplit
from future.backports.urllib.parse import parse_qs

class AfterAuthMiddleware(object):
    def process_request(self, request):
        def add_user_and_session():
            try:
                if (hasattr(request, 'user') and request.user.is_authenticated() and request.user.id):
                    uid = request.user.id
                    if uid is not None:
                        uid = str(uid)
                    request._tcell_context.user_id = uid
            except:
                pass
            if (hasattr(request, 'session') and hasattr(request.session,'session_key')):
                request._tcell_context.raw_session_id = request.session.session_key
                request._tcell_context.session_id = SanitizeUtils.hmac_half(request.session.session_key)
        safe_wrap_function("Initizating Request", add_user_and_session)

    def process_response(self, request, response):
        return response


from __future__ import unicode_literals
from __future__ import print_function

import unittest

from mock import patch
from tcell_agent.appsensor.meta import AppSensorMeta
from tcell_agent.policies.appsensor import ResponseSizeSensor

class ResponseSizeSensorTest(unittest.TestCase):

    def create_default_sensor_test(self):
        sensor = ResponseSizeSensor()
        self.assertEqual(sensor.enabled, False)
        self.assertEqual(sensor.limit, 2097152)
        self.assertEqual(sensor.excluded_route_ids, {})

    def create_enabled_sensor_test(self):
        sensor = ResponseSizeSensor({"enabled": True})
        self.assertEqual(sensor.enabled, True)
        self.assertEqual(sensor.limit, 2097152)
        self.assertEqual(sensor.excluded_route_ids, {})

    def create_sensor_with_limit_test(self):
        sensor = ResponseSizeSensor({"limit": 1024})
        self.assertEqual(sensor.enabled, False)
        self.assertEqual(sensor.limit, 1024)
        self.assertEqual(sensor.excluded_route_ids, {})

    def create_sensor_with_exclude_routes_test(self):
        sensor = ResponseSizeSensor({"exclude_routes": ["1", "10", "20"]})
        self.assertEqual(sensor.enabled, False)
        self.assertEqual(sensor.limit, 2097152)
        self.assertEqual(sensor.excluded_route_ids, {"1": True, "10": True, "20": True})

    def with_disabled_sensor_check_test(self):
        sensor = ResponseSizeSensor({"enabled": False})
        sensor.check({}, 10)

    def with_enabled_sensor_and_size_is_too_big_but_route_id_is_excluded_check_test(self):
        sensor = ResponseSizeSensor({"enabled": True, "limit": 1024, "exclude_routes": ["23947"]})
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "23947",
            session_id="session_id",
            user_id="user_id"
        )
        with patch('tcell_agent.policies.appsensor.size_sensor.sendEvent') as patched_send_event:
            sensor.check(meta, 2048)
            self.assertFalse(patched_send_event.called)

    def with_enabled_sensor_and_size_is_ok_check_test(self):
        sensor = ResponseSizeSensor({"enabled": True, "limit": 1024})
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )
        with patch('tcell_agent.policies.appsensor.size_sensor.sendEvent') as patched_send_event:
            sensor.check(meta, 10)
            self.assertFalse(patched_send_event.called)

    def with_enabled_sensor_and_size_is_too_big_check_test(self):
        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        sensor = ResponseSizeSensor({"enabled": True, "limit": 1024})

        with patch('tcell_agent.policies.appsensor.size_sensor.sendEvent') as patched_send_event:
            sensor.check(meta, 2048)
            patched_send_event.assert_called_once_with(meta, ResponseSizeSensor.DP_UNUSUAL_RESPONSE_SIZE, None, {"sz": 2048})



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function

import itertools

from tcell_agent.agent import TCellAgent, PolicyTypes
from tcell_agent.config import CONFIGURATION
import os
from tcell_agent.sensor_events import HttpTxSensorEvent, FingerprintSensorEvent, LoginSuccessfulSensorEvent, LoginFailureSensorEvent
from tcell_agent.sensor_events import RedirectSensorEvent
from tcell_agent.sensor_events import ServerAgentDetailsEvent
from .settings import send_django_setting_events
import uuid
import re

from tcell_agent.sanitize import SanitizeUtils

from future.backports.urllib.parse import urlsplit
from future.backports.urllib.parse import urlunsplit
from future.backports.urllib.parse import parse_qs
_started = False
from .routes import make_route_table
from tcell_agent.instrumentation.django.middleware.globalrequestmiddleware import GlobalRequestMiddleware
from tcell_agent.instrumentation import BaseWrapper
from tcell_agent.instrumentation.manager import InstrumentationManager

from tcell_agent.appsensor.django import django_meta
from tcell_agent.policies.appsensor.sensor import sendEvent

from logging import Logger
try:
    from django.db.models.expressions import Col
    col_supported = True
except:
    col_supported = False
from collections import defaultdict

from .. import safe_wrap_function

import logging
logger = logging.getLogger('tcell_agent').getChild(__name__)

def check_max_database_rows_reached(request, total_records, dataloss_policy, appsensor_policy):
    appsensor_meta = django_meta(request)
    if appsensor_policy:
        appsensor_policy.check_db_rows(appsensor_meta, total_records)

    if dataloss_policy and total_records > CONFIGURATION.max_data_ex_db_records_per_request:
        logger.warn("Retrieved too many records for route_id: %s", appsensor_meta.route_id)

def singlecallback(row, track_idxes, request, count):
    if count < CONFIGURATION.max_data_ex_db_records_per_request:
        if track_idxes is not None:
            for track_idx in track_idxes:
                actions, db_identifier,schema_identifier,table, column = track_idxes[track_idx]
                for action in actions:
                    try:
                        request._tcell_context.add_response_db_filter(row[track_idx], action, db_identifier, schema_identifier, table, column)
                    except Exception as e:
                        print(e)
                        pass

    return row

def multicallback2(rows, track_idxes, request, i, process_dataloss_policy):
    number_of_rows = len(rows)
    j = i * number_of_rows - 1
    for row in rows:
        j += 1
        if process_dataloss_policy:
            singlecallback(row, track_idxes, request, j)

    return number_of_rows

def multicallback(results, track_idxes, request, dataloss_policy, appsensor_policy):
    if results:
        results, results2 = itertools.tee(results)
        total_records = 0
        process_dataloss_policy = dataloss_policy and request and len(track_idxes.keys()) > 0
        for counter, rows in enumerate(results):
            total_records += multicallback2(rows, track_idxes, request, counter, process_dataloss_policy)

        safe_wrap_function(
            "Check max number of DB records",
            check_max_database_rows_reached,
            request,
            total_records,
            dataloss_policy,
            appsensor_policy
        )

        return results2

    else:
        return None

# class CursorTCellWrapper(BaseWrapper):
#     def __init__(self, instance):
#         self.cursor = instance.cursor
#         self.db = instance.db
#         super(CursorTCellWrapper,self).__init__(instance)

#     WRAP_ERROR_ATTRS = frozenset(['fetchone', 'fetchmany', 'fetchall', 'nextset'])

#     def __getattr__(self, attr):
#         print("AXY")
#         print(attr)

#         cursor_attr = getattr(self.cursor, attr)
#         if attr in CursorTCellWrapper.WRAP_ERROR_ATTRS:
#             results = self.db.wrap_database_errors(cursor_attr)
#         else:
#             results = cursor_attr
#         print(results)
#         return results

#     def __iter__(self):
#         with self.db.wrap_database_errors:
#             for item in self.cursor:
#                 yield item

#     def __enter__(self):
#         return self

#     def __exit__(self, type, value, traceback):
#         # Ticket #17671 - Close instead of passing thru to avoid backend
#         # specific behavior. Catch errors liberally because errors in cleanup
#         # code aren't useful.
#         try:
#             self.close()
#         except self.db.Database.Error:
#             pass

#     # The following methods cannot be implemented in __getattr__, because the
#     # code must run when the method is invoked, not just when it is accessed.

#     def callproc(self, procname, params=None):
#         print("proc")
#         self.db.validate_no_broken_transaction()
#         with self.db.wrap_database_errors:
#             if params is None:
#                 return self.cursor.callproc(procname)
#             else:
#                 return self.cursor.callproc(procname, params)

#     def execute(self, sql, params=None):
#         print("EONE")
#         print(type(sql).__name__)
#         self.db.validate_no_broken_transaction()
#         with self.db.wrap_database_errors:
#             if params is None:
#                 results = self.cursor.execute(sql)
#             else:
#                 results = self.cursor.execute(sql, params)
#             print(results)
#             print(self.cursor.description)
#             return results

#     def executemany(self, sql, param_list):
#         print("EMANY")
#         print(sql)
#         self.db.validate_no_broken_transaction()
#         with self.db.wrap_database_errors:
#             results = self.cursor.executemany(sql, param_list)
#             return results

    # def fetchmany(self, *args):
    #     print("EXECUTE")
    #     results = self._instance.fetchmany(*args)
    #     print("--")
    #     print(self._instance.description)
    #     print("**")
    #     print(results)
    #     return results

    #def __iter__(self):
    #    print("ITER")
    #    print(self._instance.description)
    #    for item in self._instance.__iter__():
    #        print(item)
    #       yield item

def dlp_instrumentation():

    from django.db.models.sql.compiler import SQLCompiler
    from django.db.models.sql.constants import (
        CURSOR, GET_ITERATOR_CHUNK_SIZE, MULTI, NO_RESULTS, ORDER_DIR, SINGLE,
    )

    # from django.db.backends.base.base import BaseDatabaseWrapper
    # orig_get_cursor = BaseDatabaseWrapper.cursor
    # def get_cursor(self):
    #     print("Get Cursor")
    #     print(self)
    #     #return orig_get_cursor(self)
    #     return CursorTCellWrapper(orig_get_cursor(self))
    # BaseDatabaseWrapper.cursor = get_cursor

    # def _tcell_get_columns(self, with_aliases=False):
    #     print("yy")
    #     aliases = set(self.query.extra_select.keys())
    #     print(aliases)
    #     if with_aliases:
    #         col_aliases = aliases.copy()
    #     else:
    #         col_aliases = set()
    #     print(self.query)
    #     if self.query.select:
    #         print(self.query.select)
    #         only_load = self.deferred_to_columns()
    #         for col, _ in self.query.select:
    #             if isinstance(col, (list, tuple)):
    #                 alias, column = col
    #                 print(alias)
    #                 print(column)
    #                 table = self.query.alias_map[alias].table_name
    #                 print(table)
    #                 print("---")
    #                 if table in only_load and column not in only_load[table]:
    #                     continue
    #     elif self.query.default_cols:
    #         cols, new_aliases = self.get_default_columns(with_aliases,
    #                 col_aliases)
    #         print(cols)


    def _tcell_execute_sql(_tcell_original, self, result_type=MULTI):
        results = _tcell_original(self, result_type)
        try:
            #description = self.connection.cursor().description
            request = GlobalRequestMiddleware.get_current_request()
            dataloss_policy = TCellAgent.get_policy(PolicyTypes.DATALOSS)
            appsensor_policy = TCellAgent.get_policy(PolicyTypes.APPSENSOR)
            #db = self.connection.get_connection_params().get("db")
            #db_user = self.connection.get_connection_params().get("user","_")
            #db_host = self.connection.get_connection_params().get("hosta","_")
            #db_port = self.connection.get_connection_params().get("port","_")

            if hasattr(self.connection,"vendor"):
                db_vendor = self.connection.vendor
            else:
                db_vendor = "n/a"
            try:
                db_name = self.connection.get_connection_params().get("db","n/a")
            except:
                db_name = "n/a"

            db_identifier = db_vendor
            schema_identifier = db_name

            route_id = None
            if request:
                try:
                    route_id = request._tcell_context.route_id
                except:
                    pass

            if dataloss_policy or appsensor_policy:
                if col_supported:
                    if self.select:
                        try:
                            track_idxes = {}
                            tables_track = defaultdict(set)

                            if dataloss_policy:
                                for idx, select in enumerate(self.select):
                                    try:
                                        if isinstance(select[0],Col):
                                           table = select[0].alias
                                           column = select[0].target.column
                                           tables_track[table].add(column)
                                           if results and request:
                                               actions = dataloss_policy.get_actions_for_db_field(db_identifier,schema_identifier,table,column, route_id)
                                               if (actions):
                                                    track_idxes[idx] = (actions, db_identifier,schema_identifier,table, column)
                                    except Exception as selectException:
                                        logger.debug(selectException)
                                        pass

                                for table in tables_track:
                                    TCellAgent.discover_database_fields(db_identifier, schema_identifier, table, list(tables_track[table]), route_id)

                            if result_type == MULTI:
                                ans = multicallback(results, track_idxes, request, dataloss_policy, appsensor_policy)
                                if ans:
                                    return ans


                            if results and request and len(track_idxes.keys()) > 0 and result_type == SINGLE:
                                safe_wrap_function(
                                    "Check max number of DB records",
                                    check_max_database_rows_reached,
                                    request,
                                    1,
                                    dataloss_policy,
                                    appsensor_policy
                                )
                                return singlecallback(results, track_idxes, request, 0)

                        except Exception as b:
                            logger.error("error in data-exposure mapping.")
                            logger.debug(b)
                elif self.query.select:
                    try:
                        track_idxes = {}
                        only_load = self.deferred_to_columns()
                        idx = 0
                        tables_track = defaultdict(set)

                        if dataloss_policy:
                            for col, _ in self.query.select:
                                try:
                                    if isinstance(col, (list, tuple)):
                                        table, column = col
                                        tables_track[table].add(column)
                                        if results and request:
                                            actions = dataloss_policy.get_actions_for_db_field(db_identifier,schema_identifier,table,column, route_id)
                                            if (actions):
                                                track_idxes[idx] = (actions, db_identifier,schema_identifier,table, column)
                                except Exception as selectException:
                                    logger.debug(selectException)
                                finally:
                                    idx = idx + 1
                            for table in tables_track:
                                TCellAgent.discover_database_fields(db_identifier, schema_identifier, table, list(tables_track[table]), route_id)

                        if result_type == MULTI:
                            ans = multicallback(results, track_idxes, request, dataloss_policy, appsensor_policy)
                            if ans:
                                return ans

                        if results and request and len(track_idxes.keys()) > 0 and result_type == SINGLE:
                            safe_wrap_function(
                                "Check max number of DB records",
                                check_max_database_rows_reached,
                                request,
                                1,
                                dataloss_policy,
                                appsensor_policy
                            )
                            return singlecallback(results, track_idxes, request, 0)

                    except Exception as b:
                        logger.debug("error in data-exposure mapping.")
                        logger.debug(b)
                elif self.query.default_cols:
                    try:
                        track_idxes = {}
                        opts = self.query.get_meta()
                        start_alias = self.query.get_initial_alias()
                        seen_models = {None: start_alias}
                        idx = 0
                        tables_track = defaultdict(set)

                        if dataloss_policy:
                            for field, model in opts.get_concrete_fields_with_model():
                                try:
                                    table = self.query.join_parent_model(opts, model, start_alias,
                                                                         seen_models)
                                    column = field.column
                                    tables_track[table].add(column)
                                    if results and request:
                                        actions = dataloss_policy.get_actions_for_db_field(db_identifier,schema_identifier,table,column, route_id)
                                        if (actions):
                                            track_idxes[idx] = (actions, db_identifier,schema_identifier,table, column)
                                except Exception as selectException:
                                    logger.debug(selectException)
                                    pass
                                finally:
                                    idx = idx + 1

                            for table in tables_track:
                                TCellAgent.discover_database_fields(db_identifier, schema_identifier, table, list(tables_track[table]), route_id)

                        if result_type == MULTI:
                            ans = multicallback(results, track_idxes, request, dataloss_policy, appsensor_policy)
                            if ans:
                                return ans

                        if results and request and len(track_idxes.keys()) > 0 and result_type == SINGLE:
                            safe_wrap_function(
                                "Check max number of DB records",
                                check_max_database_rows_reached,
                                request,
                                1,
                                dataloss_policy,
                                appsensor_policy
                            )
                            return singlecallback(results, track_idxes, request, 0)

                    except Exception as b:
                        logger.debug("error in data-exposure mapping.")
                        logger.debug(b)
        except Exception as wrapping_exception:
            logger.debug(wrapping_exception)
            logger.debug("Could not complete data-exposure mapping.")
        return results
    InstrumentationManager.instrument(SQLCompiler, "execute_sql", _tcell_execute_sql)

    def _tcell_log(_tcell_original, self, level, msg, args, exc_info=None, extra=None):
        # Skip us...
        if (self.name and self.name.startswith("tcell_agent")):
            return _tcell_original(self, level, msg, args, exc_info, extra)
        request = GlobalRequestMiddleware.get_current_request()
        if request:
            msg = request._tcell_context.filter_log_message(msg)
        return _tcell_original(self, level, msg, args, exc_info, extra)
    InstrumentationManager.instrument(Logger, "_log", _tcell_log)



from __future__ import unicode_literals
from __future__ import print_function

import unittest

from mock import patch
from tcell_agent.appsensor.meta import AppSensorMeta
from tcell_agent.policies.appsensor import LoginSensor
from tcell_agent.sanitize import SanitizeUtils

class LoginSensorTest(unittest.TestCase):

    def create_default_sensor_test(self):
        sensor = LoginSensor()
        self.assertEqual(sensor.enabled, False)

    def create_enabled_sensor_test(self):
        sensor = LoginSensor({"enabled": True})
        self.assertEqual(sensor.enabled, True)



    def with_disabled_sensor_check_test(self):
        sensor = LoginSensor({"enabled": False})

        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        with patch('tcell_agent.policies.appsensor.login_sensor.sendEvent') as patched_send_event:
            sensor.check(meta, 'username')
            self.assertFalse(patched_send_event.called)

    def with_no_username_sensor_check_test(self):
        sensor = LoginSensor({"enabled": True})

        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        with patch('tcell_agent.policies.appsensor.login_sensor.sendEvent') as patched_send_event:
            sensor.check(meta, None)
            patched_send_event.assert_called_once_with(meta, LoginSensor.LOGIN_FAILURE_DP, None, None, payload=None)

    def with_username_sensor_check_test(self):
        sensor = LoginSensor({"enabled": True})

        meta = AppSensorMeta(
            "remote_addr",
            "request_method",
            "abosolute_uri",
            "route_id",
            session_id="session_id",
            user_id="user_id"
        )

        with patch('tcell_agent.policies.appsensor.login_sensor.sendEvent') as patched_send_event:
            sensor.check(meta, "username")
            patched_send_event.assert_called_once_with(
                meta,
                LoginSensor.LOGIN_FAILURE_DP,
                SanitizeUtils.hmac("username"),
                None,
                payload=None
            )



from tcell_agent.policies.appsensor.sensor import sendEvent

class ResponseCodesSensor(object):
    RESPONSE_CODE_DP_DICT = {
        401:"s401",
        403:"s403",
        404:"s404",
        4:"s4xx",
        500:"s500",
        5:"s5xx"
    }

    def __init__(self, policy_json=None):
        self.enabled = False
        self.series_400_enabled = False
        self.series_500_enabled = False
        self.excluded_route_ids = {}

        if policy_json is not None:
            self.enabled = policy_json.get("enabled", False)
            self.series_400_enabled = policy_json.get("series_400_enabled", False)
            self.series_500_enabled = policy_json.get("series_500_enabled", False)

            for route_id in policy_json.get("exclude_routes", []):
                self.excluded_route_ids[route_id] = True

    def check(self, appsensor_meta, response_code):
        if not self.enabled:
            return

        if self.excluded_route_ids.get(appsensor_meta.route_id, False):
            return False

        if response_code == 200:
            return
        if not self.series_400_enabled and (response_code >= 400 and response_code < 500):
            return
        if not self.series_500_enabled and (response_code >= 500 and response_code < 600):
            return

        dp = self.RESPONSE_CODE_DP_DICT.get(response_code)
        if dp is None:
            code_series = int(response_code / 100)
            dp = self.RESPONSE_CODE_DP_DICT.get(code_series)

        if dp:
            sendEvent(
              appsensor_meta,
              dp,
              None,
              {"code": response_code})

    def __str__(self):
        return "<%s enabled: %s series_400_enabled: %s series_500_enabled: %s>" % \
            (type(self).__name__, self.enabled, self.series_400_enabled, self.series_500_enabled)



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function
import unicodedata

from tcell_agent.agent import TCellAgent, PolicyTypes
import os
from tcell_agent.sensor_events.httptx import HttpTxSensorEvent, FingerprintSensorEvent, LoginSuccessfulSensorEvent, LoginFailureSensorEvent
from tcell_agent.sensor_events.http_redirect import RedirectSensorEvent
import uuid
import re 
import datetime

from tcell_agent.instrumentation import handle_exception, safe_wrap_function
import logging
import tcell_agent.tcell_logger
LOGGER = logging.getLogger('tcell_agent').getChild(__name__)

class TimerMiddleware(object):

    def process_request(self, request):
        def start():
            request._tcell_context.start_time = datetime.datetime.now()#
        safe_wrap_function("Start Request Timer", start)        

    def process_response(self, request, response):
        def stop():
            endtime = datetime.datetime.now()
            if request._tcell_context.start_time != 0:
                request_time = int((endtime - request._tcell_context.start_time).total_seconds() * 1000)
                TCellAgent.request_metric(
                    request._tcell_context.route_id, 
                    request_time, 
                    request._tcell_context.remote_addr,
                    request._tcell_context.user_agent,
                    request._tcell_context.session_id,
                    request._tcell_context.user_id
                )
                LOGGER.debug("request took {time}".format(time=request_time))
        safe_wrap_function("Stop Request Timer", stop)
        return response



from tcell_agent.agent import TCellAgent, PolicyTypes
from tcell_agent.appsensor.django import django_meta
from tcell_agent.instrumentation.django.middleware.globalrequestmiddleware import GlobalRequestMiddleware
from tcell_agent.instrumentation.manager import InstrumentationManager
from tcell_agent.instrumentation.django.utils import django15or16

import logging
import tcell_agent.tcell_logger

LOGGER = logging.getLogger('tcell_agent').getChild(__name__)

def instrument_database_error_wrapper():
    if django15or16 == False:
        # Todo: Add Django 1.5/1.6 support
        try:
            from django.db.utils import DatabaseErrorWrapper
            def _tcell_exit(_tcell_original_exit, self, exc_type, exc_value, traceback):
                if exc_type is not None:
                    appsensor_policy = TCellAgent.get_policy(PolicyTypes.APPSENSOR)
                    if appsensor_policy is not None:
                        request = GlobalRequestMiddleware.get_current_request()
                        meta = django_meta(request)
                        appsensor_policy.sql_exception_detected(self.wrapper.Database, meta, exc_type, exc_value, traceback)

                return _tcell_original_exit(self, exc_type, exc_value, traceback)
            InstrumentationManager.instrument(DatabaseErrorWrapper, "__exit__", _tcell_exit)
        except Exception as e:
            LOGGER.exception("Could not instrument database error wrapper", e)



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function


class TCellPolicy(object):
    def __init__(self):
        self.policy_id = None



# -*- coding: utf-8 -*-
# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function

from tcell_agent.policies import TCellPolicy

import logging
logger = logging.getLogger('tcell_agent').getChild(__name__)

class SecureHeader:
    approved_headers = [
                    "strict-transport-security",
                    "x-frame-options",
                    "x-xss-protection",
                    "x-content-type-options",
                    "x-permitted-cross-domain-policies",
                    "x-download-options"
    ]
    def __init__(self, name, value):
        if name.lower() not in self.approved_headers:
            raise Exception("Header name is not in approved list: " + name)
        self.name = name
        self.value = value

    def header_value(self):
        return self.value

class SecureHeadersPolicy(TCellPolicy):
    def __init__(self, policy_json=None):
        super(SecureHeadersPolicy, self).__init__()
        self.secure_headers = []
        if policy_json is not None:
            self.loadFromJson(policy_json)

    def headers(self):
        headers = []
        for header in self.secure_headers:
            headers.append([header.name, header.value])
        return headers

    def loadFromJson(self, policy_json):
        if "policy_id" in policy_json:
            self.policy_id = policy_json["policy_id"]
        else:
            raise Exception("Policy Id Not Found")
        secure_headers = []
        if "headers" in policy_json:
            for header in policy_json["headers"]:
                if "name" in header and "value" in header:
                    try:
                        secure_header = SecureHeader(header["name"], header["value"])
                        secure_headers.append(secure_header)
                    except:
                        logger.error("error parsing secure header (%s, %s)" % (header["name"], header["value"]))
        self.secure_headers = secure_headers
        




import tcell_agent.agent

from tcell_agent.sensor_events import AppSensorEvent

def sendEvent(
        appsensor_meta,
        detection_point,
        parameter,
        meta,
        payload=None,
        pattern=None):
    tcell_agent.agent.TCellAgent.send(AppSensorEvent(
        detection_point,
        parameter,
        appsensor_meta.location,
        appsensor_meta.remote_address,
        appsensor_meta.route_id,
        meta,
        appsensor_meta.method,
        payload=payload,
        user_id=appsensor_meta.user_id,
        hmacd_session_id=appsensor_meta.session_id,
        pattern=pattern
    ))



# Copyright (C) 2015 tCell.io, Inc. - All Rights Reserved

from __future__ import unicode_literals
from __future__ import print_function
from . import SensorEvent

class AppRouteSensorEvent(SensorEvent):
  
    def __init__(self, uri, method, destination, rid, params=None):
        super(AppRouteSensorEvent, self).__init__("appserver_routes", ensure_delivery=True, queue_wait=True)
        self["uri"] = uri
        self["destination"] = destination
        self["method"] = method
        self["rid"] = rid
        if params is not None:
            self["params"] = params